//=============================================================================
// (c) Copyright 2005 Diamond Systems Corporation. Use of this source code
// is subject to the terms of Diamond Systems' Software License Agreement.
// Diamond Systems provides no warranty of proper performance if this
// source code is modified.
//
// File: DSCDIOFunctions.c   v5.9 
// Desc: Sample program that demonstrates how to use the digital I/O functions
// Created by	KL
//=============================================================================

#include <stdio.h>

#ifdef _WIN32
#ifndef _WIN32_WCE
#include <conio.h>
#endif

#include <windows.h>
#include <stdlib.h>
#include <math.h>
// diamond driver includes
#include "dscud.h"
#endif

// DOS
#ifdef __BORLANDC__
#include <conio.h>
#include <dos.h>
#include <stdlib.h>
#include <math.h>
// diamond driver includes
#include "../../../../current/dev/source/dscud.h"
#endif

// Linux and QNX
#if defined(linux) || defined(__QNXNTO__) || defined(_WRS_VXWORKS_5_X)
#include <stdlib.h>
#include <math.h>
#include <time.h>
// diamond driver includes
#include "dscud.h"


#ifdef _WRS_VXWORKS_5_X
#include <selectLib.h>
#define main OMMDIODSCDIOFunctions
#else
#include <sys/time.h>
#endif

static int kbhit()
{
	struct timeval timeout;
	fd_set rfds;

	timeout.tv_sec = 0;
	timeout.tv_usec = 0;

	FD_ZERO(&rfds);
	FD_SET(0, &rfds);

	if ( select(0+1, &rfds, NULL, NULL, &timeout) > 0 )
		return getchar();

	return 0;
}
#endif


// var declarations
BYTE result;	// returned error code
DSCB dscb;		// handle used to refer to the board
DSCCB dsccb;	// structure containing board settings
BYTE portOut;	// port used for digital I/O output
BYTE portIn;	// port used for digital I/O input
BYTE output_b;	// BYTE sent to digital output
BYTE input_b;	// BYTE receiving digital input
BYTE config_bytes[2];   // holds configuration for DIO ports
ERRPARAMS errorParams;  // structure for returning error code and error string
int intBuff;            // temp variable of size int

//=============================================================================
// Name: main()
// Desc: Starting function that calls the driver functions used
//
//		 NOTE: By convention, you should capture the BYTE return value for each
//		 driver API call, and check the error code.
//
//	     I. Driver Initialization
//	    II. Board Initialization
//	   III. DIO Input and Output
//	    IV. Cleanup
//
//=============================================================================

int main( void )
{
	//=========================================================================
	// I. DRIVER INITIALIZATION
	//
	//    Initializes the DSCUD library.
	//
	//    Initialize the driver, using the driver version for validation
	//
	//=========================================================================

	if( dscInit( DSC_VERSION ) != DE_NONE )
	{
		dscGetLastError(&errorParams);
		fprintf( stderr, "dscInit error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );	
		return 0;
	}

	//=========================================================================
	// II. BOARD INITIALIZATION
	//
	//	   Initialize the OMMDIO board. This function passes the various
	//	   hardware parameters to the driver and resets the hardware.
	//
	//=========================================================================

	printf( "\nOMMDIO BOARD INITIALIZATION:\n" );

	printf("Enter the base address (default 0x300) : ");
	scanf("%hx", &dsccb.base_address );

	if(dscInitBoard(DSC_OMMDIO, &dsccb, &dscb)!= DE_NONE)
	{
		dscGetLastError(&errorParams);
		fprintf( stderr, "dscInitBoard error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );	
		return 0;
	}

	//=========================================================================
	// III. DIO INPUT AND OUTPUT
	//
	//		Perform a set of byte-level inputs and outputs. 
	//		For word and bit-level operations, the steps are nearly
	//		identical with a few minor changes to function names and
	//		the size of the data to be transferred.
	//
	//		Setup the following parameters for the user interrupt:
	//			config_bytes[]  for dscDIOSetConfig
	//			portOut
	//			portIn
	//			output_b;
	//
	//=========================================================================

	printf( "\nDIO INPUT AND OUTPUT:\n" );

	/*
		config_bytes[0] = 0x82;  // set the mode of the dio ports to input and output
							  // refer to the manual concerning the values
	*/

	printf("Enter the configuration byte for channels 0-2 (default 0x82) : ");
	scanf( "%x", &intBuff);
	config_bytes[0] = (BYTE) intBuff;

	printf("Enter the configuration byte for channels 3-5 (default 0x82) : ");
	scanf( "%x", &intBuff);
	config_bytes[1] = (BYTE) intBuff;

	dscDIOSetConfig(dscb, config_bytes);	// setupt input/output modes for dio ports

	do
	{
		printf("Enter the output port : "); // port to output values to
		scanf( "%d", &intBuff);
		portOut = (BYTE) intBuff;

		printf("Enter the input port : ");  // port to retrieve values from
		scanf( "%d", &intBuff);
		portIn = (BYTE) intBuff;
		
		printf("Enter the value to port : ");	// byte to output
		scanf( "%d", &intBuff);
		output_b = (BYTE) intBuff;

		getchar();

		// output byte
		if( ( result = dscDIOOutputByte( dscb, portOut, (BYTE)( output_b ) ) ) != DE_NONE )
		{
			dscGetLastError(&errorParams);
			fprintf( stderr, "dscDIOOutputByte error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );	
			return 0;
		}

		// input and retrieve the value
		if( ( result = dscDIOInputByte( dscb, portIn, &input_b ) ) != DE_NONE )
		{
			dscGetLastError(&errorParams);
			fprintf( stderr, "dscDIOInputByte error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );	
			return 0;
		}

		printf( "Byte value sent to port: %d, Byte value received from port: %d\n", (short)output_b, (short)input_b );  
		printf("\nPress 'q' to exit or ENTER to continue \n");
	}while( getchar() != 'q' );	
	

	//=========================================================================
	// IV. CLEANUP
	//
	//	   Cleanup any remnants left by the program and free the resources used
	//	   by the driver.
	//
	//     STEPS TO FOLLOW:
	//
	//	   1. free the driver resources
	//=========================================================================

	dscFree();

	printf( "\nDSCDIOFunctions completed.\n" );

	return 0;
} // end main()


#ifdef _WIN32_WCE
int WINAPI WinMain(	HINSTANCE hInstance,
					HINSTANCE hPrevInstance,
					LPTSTR    lpCmdLine,
					int       nCmdShow)
{
	main();

	return 0;
}

#endif
